@@ -34,6 +34,7 @@ public:
 		Dictionary,
 		Function,
 		Structure,
+		Reference
 	};
 
 	/// @name Destructor
@@ -44,6 +45,7 @@ public:
 	/// @name Getter methods
 	/// @{
 	const std::string& getName() const { return _name; }
+	const std::string& getDocumentation() const { return _documentation; }
 	ExpressionType getDataType() const { return _dataType; }
 	Symbol::Type getType() const { return _type; }
 	TokenType getTokenType() const
@@ -55,6 +57,7 @@ public:
 			case Type::Dictionary : return TokenType::DICTIONARY_SYMBOL;
 			case Type::Function : return TokenType::FUNCTION_SYMBOL;
 			case Type::Structure : return TokenType::STRUCTURE_SYMBOL;
+			case Type::Reference : return TokenType::REFERENCE_SYMBOL;
 			default: return TokenType::INVALID;
 		}
 	}
@@ -73,17 +76,21 @@ public:
 	bool isDictionary() const { return _type == Symbol::Type::Dictionary; }
 	bool isFunction() const { return _type == Symbol::Type::Function; }
 	bool isStructure() const { return _type == Symbol::Type::Structure; }
+	bool isReference() const { return _type == Symbol::Type::Reference; }
 	/// @}
 
 protected:
 	/// @name Constructors
 	/// @{
-	Symbol(Symbol::Type type, const std::string& name, ExpressionType dataType)
-		: _type(type), _name(name), _dataType(dataType) {}
+	Symbol(Symbol::Type type, const std::string& name, ExpressionType dataType, const std::string& documentation = "")
+		: _type(type), _name(name), _documentation(documentation), _dataType(dataType)
+	{
+	}
 	/// @}
 
 	Symbol::Type _type; ///< Type of the symbol
 	std::string _name; ///< Name
+	std::string _documentation; ///< Documentation
 	ExpressionType _dataType; ///< Data type of the symbol
 };
 
