@@ -7,6 +7,7 @@
 #pragma once
 
 namespace yaramod {
+
 /**
  * Represents type of parsed tokens.
  */
@@ -64,6 +65,7 @@ enum class TokenType
 	GLOBAL,
 	NONE,
 	RULE,
+	VARIABLES,
 	STRINGS,
 	CONDITION,
 	ASCII,
@@ -72,6 +74,8 @@ enum class TokenType
 	FULLWORD,
 	PRIVATE_STRING_MODIFIER,
 	XOR,
+	BASE64,
+	BASE64WIDE,
 	IMPORT_MODULE,
 	IMPORT_KEYWORD,
 	NOT,
@@ -118,14 +122,18 @@ enum class TokenType
 	UNARY_MINUS,
 	META_KEY,
 	META_VALUE,
+	VARIABLE_KEY,
 	STRING_KEY,
 	VALUE_SYMBOL,
 	FUNCTION_SYMBOL,
 	ARRAY_SYMBOL,
 	DICTIONARY_SYMBOL,
 	STRUCTURE_SYMBOL,
+	REFERENCE_SYMBOL,
 	LP_ENUMERATION,
 	RP_ENUMERATION,
+	LSQB_ENUMERATION,
+	RSQB_ENUMERATION,
 	LP_WITH_SPACE_AFTER,
 	RP_WITH_SPACE_BEFORE,
 	LP_WITH_SPACES,
