@@ -8,7 +8,8 @@
 
 #include <vector>
 
-#include "yaramod/types/modules/modules_pool.h"
+#include "yaramod/types/features.h"
+#include "yaramod/types/modules/module_pool.h"
 #include "yaramod/types/rule.h"
 
 namespace yaramod {
@@ -22,8 +23,8 @@ class YaraFile
 public:
 	/// @name Constructors
 	/// @{
-	YaraFile(ImportFeatures features = ImportFeatures::AllCurrent);
-	YaraFile(const std::shared_ptr<TokenStream>& tokenStream, ImportFeatures features = ImportFeatures::AllCurrent);
+	YaraFile(Features features = Features::AllCurrent);
+	YaraFile(const std::shared_ptr<TokenStream>& tokenStream, Features features = Features::AllCurrent);
 	YaraFile(YaraFile&&) noexcept;
 
 	YaraFile& operator=(YaraFile&&) noexcept;
@@ -37,12 +38,12 @@ public:
 
 	/// @name Addition methods
 	/// @{
-	bool addImport(TokenIt import, ModulesPool& modules);
+	bool addImport(TokenIt import, ModulePool& modules);
+	bool addImports(const std::vector<TokenIt>& imports, ModulePool& modules);
 	void addRule(Rule&& rule, bool extractTokens = false);
 	void addRule(std::unique_ptr<Rule>&& rule, bool extractTokens = false);
 	void addRule(const std::shared_ptr<Rule>& rule, bool extractTokens = false);
 	void addRules(const std::vector<std::shared_ptr<Rule>>& rules, bool extractTokens = false);
-	bool addImports(const std::vector<TokenIt>& imports, ModulesPool& modules);
 	void insertRule(std::size_t position, std::unique_ptr<Rule>&& rule);
 	void insertRule(std::size_t position, const std::shared_ptr<Rule>& rule);
 	/// @}
@@ -102,7 +103,7 @@ private:
 	std::unordered_map<std::string, Module*> _importTable;
 	std::unordered_map<std::string, Rule*> _ruleTable;
 
-	ImportFeatures _importFeatures; ///< Determines which symbols are needed
+	Features _Features; ///< Determines which symbols are needed
 	std::vector<std::shared_ptr<Symbol>> _vtSymbols; ///< Virust Total symbols
 };
 
