@@ -1,7 +1,7 @@
 /**
  * @file src/yaramod.h
  * @brief Declaration of yaramod interface.
- * @copyright (c) 2017 Avast Software, licensed under the MIT license
+ * @copyright (c) 2019 Avast Software, licensed under the MIT license
  */
 
 #pragma once
@@ -10,13 +10,13 @@
 #define STR(x) STR_HELPER(x)
 
 #define YARAMOD_VERSION_MAJOR 3
-#define YARAMOD_VERSION_MINOR 7
-#define YARAMOD_VERSION_PATCH 4
+#define YARAMOD_VERSION_MINOR 10
+#define YARAMOD_VERSION_PATCH 1
 #define YARAMOD_VERSION_ADDEND ""
 
 #define YARAMOD_VERSION STR(YARAMOD_VERSION_MAJOR) "." STR(YARAMOD_VERSION_MINOR) "." STR(YARAMOD_VERSION_PATCH) YARAMOD_VERSION_ADDEND
 
-#define YARA_SYNTAX_VERSION "3.11"
+#define YARA_SYNTAX_VERSION "4.0"
 
 #include <memory>
 
@@ -30,18 +30,19 @@ class Yaramod
 {
 public:
 	/*
-	 * @param ParserMode
-	 * Regular -- regular YARA parser
-	 * IncludeGuarded -- protection against inclusion of the same file multiple times
+	 * Constructor
 	 *
 	 * @param features determines iff we want to use aditional Avast-specific symbols or VirusTotal-specific symbols in the imported modules
+	 * @param moduleDirectory determines a directory for additional YARA modules to be added
 	 */
-	Yaramod(ImportFeatures features = ImportFeatures::AllCurrent) : _driver(features) {}
+	Yaramod(Features features = Features::AllCurrent, const std::string& moduleDirectory = "") : _driver(features, moduleDirectory)	{}
 	/**
 	 * Parses file at given path.
 	 *
 	 * @param filePath Path to the file.
 	 * @param parserMode Parsing mode.
+	 *   - Regular -- regular YARA parser
+	 *   - IncludeGuarded -- protection against inclusion of the same file multiple times
 	 *
 	 * @return Valid @c YaraFile instance if parsing succeeded, otherwise @c nullptr.
 	 */
@@ -51,6 +52,8 @@ public:
 	 *
 	 * @param inputStream Input stream.
 	 * @param parserMode Parsing mode.
+	 *   - Regular -- regular YARA parser
+	 *   - IncludeGuarded -- protection against inclusion of the same file multiple times
 	 *
 	 * @return Valid @c YaraFile instance if parsing succeeded, otherwise @c nullptr.
 	 */
@@ -58,6 +61,13 @@ public:
 
 	const YaraFile& getParsedFile() const;
 
+	/**
+	 * Returns ModulePool used in the parser, which gives information on which modules are available.
+	 *
+	 * @return Used ModulePool
+	 */
+	std::map<std::string, Module*> getModules() const;
+
 private:
 	ParserDriver _driver;
 };
