@@ -20,9 +20,9 @@ namespace yaramod {
  */
 std::unique_ptr<YaraFile> YaraFileBuilder::get(bool recheck, ParserDriver* external_driver)
 {
-	auto yaraFile = std::make_unique<YaraFile>(std::move(_tokenStream), _import_features);
+	auto yaraFile = std::make_unique<YaraFile>(std::move(_tokenStream), _features);
 	for (const auto& module_token : _module_tokens)
-		yaraFile->addImport(module_token.second, _modules_pool);
+		yaraFile->addImport(module_token.second, _module_pool);
 	yaraFile->addRules(_rules);
 
 	_module_tokens.clear();
@@ -52,7 +52,7 @@ std::unique_ptr<YaraFile> YaraFileBuilder::get(bool recheck, ParserDriver* exter
 		}
 		else
 		{
-			ParserDriver driver(ImportFeatures::AllCurrent);
+			ParserDriver driver(Features::AllCurrent);
 			try
 			{
 				driver.parse(ss);
