@@ -65,14 +65,14 @@ void addEnums(py::module& module)
 		.value("Regular", ParserMode::Regular)
 		.value("IncludeGuarded", ParserMode::IncludeGuarded);
 
-	py::enum_<ImportFeatures>(module, "ImportFeatures", py::arithmetic())
-		.value("Basic", ImportFeatures::Basic)
-		.value("AvastOnly", ImportFeatures::AvastOnly)
-		.value("VirusTotalOnly", ImportFeatures::VirusTotalOnly)
-		.value("Avast", ImportFeatures::Avast)
-		.value("VirusTotal", ImportFeatures::VirusTotal)
-		.value("AllCurrent", ImportFeatures::AllCurrent)
-		.value("Everything", ImportFeatures::Everything);
+	py::enum_<Features>(module, "Features", py::arithmetic())
+		.value("Basic", Features::Basic)
+		.value("AvastOnly", Features::AvastOnly)
+		.value("VirusTotalOnly", Features::VirusTotalOnly)
+		.value("Avast", Features::Avast)
+		.value("VirusTotal", Features::VirusTotal)
+		.value("AllCurrent", Features::AllCurrent)
+		.value("Everything", Features::Everything);
 
 	py::enum_<IntMultiplier>(module, "IntMultiplier")
 		.value("Empty", IntMultiplier::None)
@@ -96,7 +96,9 @@ void addEnums(py::module& module)
 		.value("Fullword", StringModifier::Type::Fullword)
 		.value("Nocase", StringModifier::Type::Nocase)
 		.value("Private", StringModifier::Type::Private)
-		.value("Xor", StringModifier::Type::Xor);
+		.value("Xor", StringModifier::Type::Xor)
+		.value("Base64", StringModifier::Type::Base64)
+		.value("Base64Wide", StringModifier::Type::Base64Wide);
 
 	py::enum_<Expression::Type>(module, "ExpressionType")
 		.value("Undefined", Expression::Type::Undefined)
@@ -163,6 +165,7 @@ void addEnums(py::module& module)
 		.value("Global", TokenType::GLOBAL)
 		.value("None", TokenType::NONE)
 		.value("Rule", TokenType::RULE)
+		.value("Variables", TokenType::VARIABLES)
 		.value("Strings", TokenType::STRINGS)
 		.value("Condition", TokenType::CONDITION)
 		.value("Ascii", TokenType::ASCII)
@@ -171,6 +174,8 @@ void addEnums(py::module& module)
 		.value("Fullword", TokenType::FULLWORD)
 		.value("PrivateStringModifier", TokenType::PRIVATE_STRING_MODIFIER)
 		.value("Xor", TokenType::XOR)
+		.value("Base64", TokenType::BASE64)
+		.value("Base64Wide", TokenType::BASE64WIDE)
 		.value("ImportModule", TokenType::IMPORT_MODULE)
 		.value("ImportKeyword", TokenType::IMPORT_KEYWORD)
 		.value("Not", TokenType::NOT)
@@ -217,6 +222,7 @@ void addEnums(py::module& module)
 		.value("UnaryMinus", TokenType::UNARY_MINUS)
 		.value("MetaKey", TokenType::META_KEY)
 		.value("MetaValue", TokenType::META_VALUE)
+		.value("VariableKey", TokenType::VARIABLE_KEY)
 		.value("StringKey", TokenType::STRING_KEY)
 		.value("ValueSymbol", TokenType::VALUE_SYMBOL)
 		.value("FunctionSymbol", TokenType::FUNCTION_SYMBOL)
@@ -276,6 +282,7 @@ void addBasicClasses(py::module& module)
 		.def_property_readonly("text", &Rule::getText)
 		.def_property("name", &Rule::getName, &Rule::setName)
 		.def_property("metas", py::overload_cast<>(&Rule::getMetas), &Rule::setMetas, py::return_value_policy::reference)
+		.def_property("variables", py::overload_cast<>(&Rule::getVariables), &Rule::setVariables, py::return_value_policy::reference)
 		.def_property("tags", &Rule::getTags, &Rule::setTags)
 		.def_property("modifier", &Rule::getModifier, &Rule::setModifier)
 		.def_property_readonly("strings", &Rule::getStrings, py::return_value_policy::reference)
@@ -299,7 +306,17 @@ void addBasicClasses(py::module& module)
 
 	py::class_<Meta>(module, "Meta")
 		.def_property("key", &Meta::getKey, &Meta::setKey)
-		.def_property("value", &Meta::getValue, &Meta::setValue);
+		.def_property("value", &Meta::getValue, &Meta::setValue)
+		.def_property_readonly("token_key", [](Meta& self) {
+				return *self.getKeyTokenIt();
+			})
+		.def_property_readonly("token_value", [](Meta& self) {
+				return *self.getValueTokenIt();
+			});
+
+	py::class_<Variable>(module, "Variable")
+		.def_property("key", &Variable::getKey, &Variable::setKey)
+		.def_property("value", &Variable::getValue, &Variable::setValue);
 
 	py::class_<Literal>(module, "Literal")
 		.def(py::init<const std::string&>())
@@ -326,7 +343,8 @@ void addBasicClasses(py::module& module)
 		.def_property_readonly("symbol", &Literal::getSymbol);
 
 	py::class_<Module, std::shared_ptr<Module>>(module, "Module")
-		.def_property_readonly("name", &Module::getName);
+		.def_property_readonly("name", &Module::getName)
+		.def_property_readonly("structure", &Module::getStructure);
 
 	py::class_<String, std::shared_ptr<String>>(module, "String")
 		.def_property_readonly("text", &String::getText)
@@ -344,6 +362,8 @@ void addBasicClasses(py::module& module)
 		.def_property_readonly("is_nocase", &String::isNocase)
 		.def_property_readonly("is_private", &String::isPrivate)
 		.def_property_readonly("is_xor", &String::isXor)
+		.def_property_readonly("is_base64", &String::isBase64)
+		.def_property_readonly("is_base64_wide", &String::isBase64Wide)
 		.def_property_readonly("location", &String::getLocation)
 		.def_property_readonly("modifiers_text", &String::getModifiersText)
 		.def_property_readonly("token_id", &String::getIdentifierToken)
@@ -372,6 +392,8 @@ void addBasicClasses(py::module& module)
 		.def_property_readonly("is_nocase", &StringModifier::isNocase)
 		.def_property_readonly("is_private", &StringModifier::isPrivate)
 		.def_property_readonly("is_xor", &StringModifier::isXor)
+		.def_property_readonly("is_base64", &StringModifier::isBase64)
+		.def_property_readonly("is_base64_wide", &StringModifier::isBase64Wide)
 		.def_property_readonly("text", &StringModifier::getText);
 
 	py::class_<AsciiStringModifier, StringModifier, std::shared_ptr<AsciiStringModifier>>(module, "AsciiStringModifier");
@@ -382,6 +404,10 @@ void addBasicClasses(py::module& module)
 	py::class_<XorStringModifier, StringModifier, std::shared_ptr<XorStringModifier>>(module, "XorStringModifier")
 		.def_property_readonly("is_range", &XorStringModifier::isRange)
 		.def_property_readonly("is_single_key", &XorStringModifier::isSingleKey);
+	py::class_<Base64StringModifier, StringModifier, std::shared_ptr<Base64StringModifier>>(module, "Base64StringModifier")
+		.def_property_readonly("has_alphabet", &Base64StringModifier::hasAlphabet);
+	py::class_<Base64WideStringModifier, StringModifier, std::shared_ptr<Base64WideStringModifier>>(module, "Base64WideStringModifier")
+		.def_property_readonly("has_alphabet", &Base64WideStringModifier::hasAlphabet);
 
 	py::class_<RegexpUnit, std::shared_ptr<RegexpUnit>>(module, "RegexpUnit")
 		.def("accept", &RegexpUnit::accept)
@@ -431,21 +457,34 @@ void addBasicClasses(py::module& module)
 
 	py::class_<Symbol, std::shared_ptr<Symbol>>(module, "Symbol")
 		.def_property_readonly("name", &Symbol::getName)
+		.def_property_readonly("documentation", &Symbol::getDocumentation)
 		.def_property_readonly("data_type", &Symbol::getDataType)
 		.def_property_readonly("is_value", &Symbol::isValue)
 		.def_property_readonly("is_array", &Symbol::isArray)
 		.def_property_readonly("is_dictionary", &Symbol::isDictionary)
 		.def_property_readonly("is_function", &Symbol::isFunction)
-		.def_property_readonly("is_structure", &Symbol::isStructure);
+		.def_property_readonly("is_structure", &Symbol::isStructure)
+		.def_property_readonly("is_reference", &Symbol::isReference);
 
 	py::class_<ValueSymbol, Symbol, std::shared_ptr<ValueSymbol>>(module, "ValueSymbol");
-	py::class_<ArraySymbol, Symbol, std::shared_ptr<ArraySymbol>>(module, "ArraySymbol");
-	py::class_<DictionarySymbol, Symbol, std::shared_ptr<DictionarySymbol>>(module, "DictionarySymbol");
-	py::class_<FunctionSymbol, Symbol, std::shared_ptr<FunctionSymbol>>(module, "FunctionSymbol");
+	py::class_<ArraySymbol, Symbol, std::shared_ptr<ArraySymbol>>(module, "ArraySymbol")
+		.def_property_readonly("element_type", &ArraySymbol::getElementType)
+		.def_property_readonly("structure", &ArraySymbol::getStructuredElementType);
+	py::class_<DictionarySymbol, Symbol, std::shared_ptr<DictionarySymbol>>(module, "DictionarySymbol")
+		.def_property_readonly("element_type", &DictionarySymbol::getElementType)
+		.def_property_readonly("structure", &DictionarySymbol::getStructuredElementType);
+	py::class_<FunctionSymbol, Symbol, std::shared_ptr<FunctionSymbol>>(module, "FunctionSymbol")
+		.def_property_readonly("return_type", &FunctionSymbol::getReturnType)
+		.def_property_readonly("overloads", &FunctionSymbol::getAllOverloads)
+		.def_property_readonly("documentations", &FunctionSymbol::getAllDocumentations)
+		.def_property_readonly("argument_names", &FunctionSymbol::getAllArgumentNames);
 	py::class_<StructureSymbol, Symbol, std::shared_ptr<StructureSymbol>>(module, "StructureSymbol")
+		.def_property_readonly("attributes", &StructureSymbol::getAttributes)
 		.def("get_attribute", [](const StructureSymbol& self, const std::string& name) {
 				return self.getAttribute(name).value_or(nullptr);
 			});
+	py::class_<ReferenceSymbol, Symbol, std::shared_ptr<ReferenceSymbol>>(module, "ReferenceSymbol")
+		.def_property_readonly("symbol", &ReferenceSymbol::getSymbol);
 }
 
 void addTokenStreamClass(py::module& module)
@@ -574,19 +613,31 @@ void addExpressionClasses(py::module& module)
 		.def_property("variable",
 				&ForExpression::getVariable,
 				py::overload_cast<const Expression::Ptr&>(&ForExpression::setVariable))
-		.def_property("iterated_set",
-				&ForExpression::getIteratedSet,
-				py::overload_cast<const Expression::Ptr&>(&ForExpression::setIteratedSet))
+		.def_property("iterable",
+				&ForExpression::getIterable,
+				py::overload_cast<const Expression::Ptr&>(&ForExpression::setIterable))
 		.def_property("body",
 				&ForExpression::getBody,
 				py::overload_cast<const Expression::Ptr&>(&ForExpression::setBody));
-	exprClass<ForIntExpression, ForExpression>(module, "ForIntExpression")
+	exprClass<ForDictExpression, ForExpression>(module, "ForDictExpression")
+		.def_property("id1",
+				&ForDictExpression::getId1,
+				py::overload_cast<const std::string&>(&ForDictExpression::setId1))
+		.def_property("id2",
+				&ForDictExpression::getId2,
+				py::overload_cast<const std::string&>(&ForDictExpression::setId2));
+	exprClass<ForArrayExpression, ForExpression>(module, "ForArrayExpression")
 		.def_property("id",
-				&ForIntExpression::getId,
-				py::overload_cast<const std::string&>(&ForIntExpression::setId));
+				&ForArrayExpression::getId,
+				py::overload_cast<const std::string&>(&ForArrayExpression::setId));
 	exprClass<ForStringExpression, ForExpression>(module, "ForStringExpression");
 	exprClass<OfExpression, ForExpression>(module, "OfExpression");
 
+	exprClass<IterableExpression>(module, "IterableExpression")
+		.def_property("elements",
+				&IterableExpression::getElements,
+				py::overload_cast<const std::vector<Expression::Ptr>&>(&IterableExpression::setElements));
+
 	exprClass<SetExpression>(module, "SetExpression")
 		.def_property("elements",
 				&SetExpression::getElements,
@@ -662,7 +713,7 @@ void addExpressionClasses(py::module& module)
 void addBuilderClasses(py::module& module)
 {
 	py::class_<YaraFileBuilder>(module, "YaraFileBuilder")
-		.def(py::init<ImportFeatures>(), py::arg("import_features") = ImportFeatures::AllCurrent)
+		.def(py::init<Features, const std::string&>(), py::arg("import_features") = Features::AllCurrent, py::arg("modules_directory") = "")
 		.def("get", [](YaraFileBuilder& self, bool recheck) {
 				return self.get(recheck, nullptr);
 			}, py::arg("recheck") = false)
@@ -686,6 +737,13 @@ void addBuilderClasses(py::module& module)
 		.def("with_uint_meta", &YaraRuleBuilder::withUIntMeta)
 		.def("with_hex_int_meta", &YaraRuleBuilder::withHexIntMeta)
 		.def("with_bool_meta", &YaraRuleBuilder::withBoolMeta)
+		.def("with_string_variable", &YaraRuleBuilder::withStringVariable)
+		.def("with_int_variable", &YaraRuleBuilder::withIntVariable)
+		.def("with_uint_variable", &YaraRuleBuilder::withUIntVariable)
+		.def("with_hex_int_variable", &YaraRuleBuilder::withHexIntVariable)
+		.def("with_double_variable", &YaraRuleBuilder::withDoubleVariable)
+		.def("with_bool_variable", &YaraRuleBuilder::withBoolVariable)
+		.def("with_struct_variable", &YaraRuleBuilder::withStructVariable)
 		.def("with_plain_string", &YaraRuleBuilder::withPlainString, py::arg("id"), py::arg("value"))
 		.def("with_hex_string", &YaraRuleBuilder::withHexString)
 		.def("with_regexp", &YaraRuleBuilder::withRegexp, py::arg("id"), py::arg("value"), py::arg("suffix_mods") = std::string{})
@@ -703,6 +761,20 @@ void addBuilderClasses(py::module& module)
 			else if (args.size() == 2)
 				return self.xor_(args[0].cast<int>(), args[1].cast<int>());
 			throw std::invalid_argument("xor() expects either 0, 1 or 2 arguments");
+		})
+		.def("base64", [](YaraRuleBuilder& self, py::args args) {
+			if (args.size() == 0)
+				return self.base64();
+			else if (args.size() == 1)
+				return self.base64(args[0].cast<std::string>());
+			throw std::invalid_argument("base64() expects either 0 or 1 argument");
+		})
+		.def("base64wide", [](YaraRuleBuilder& self, py::args args) {
+			if (args.size() == 0)
+				return self.base64wide();
+			else if (args.size() == 1)
+				return self.base64wide(args[0].cast<std::string>());
+			throw std::invalid_argument("base64wide() expects either 0 or 1 argument");
 		});
 
 	py::class_<YaraExpressionBuilder>(module, "YaraExpressionBuilder")
@@ -770,6 +842,13 @@ void addBuilderClasses(py::module& module)
 	module.def("match_at", py::overload_cast<const std::string&, const YaraExpressionBuilder&>(&matchAt));
 	module.def("match_in_range", py::overload_cast<const std::string&, const YaraExpressionBuilder&>(&matchInRange));
 
+	module.def("for_loop", py::overload_cast<
+			const YaraExpressionBuilder&,
+			const std::string&,
+			const std::string&,
+			const YaraExpressionBuilder&,
+			const YaraExpressionBuilder&
+		>(&forLoop));
 	module.def("for_loop", py::overload_cast<
 			const YaraExpressionBuilder&,
 			const std::string&,
@@ -783,6 +862,8 @@ void addBuilderClasses(py::module& module)
 		>(&forLoop));
 	module.def("of", &of);
 
+	module.def("iterable", &iterable);
+
 	module.def("set", &set);
 	module.def("range", &range);
 
@@ -827,13 +908,14 @@ void addBuilderClasses(py::module& module)
 void addMainClass(py::module& module)
 {
 	py::class_<Yaramod>(module, "Yaramod")
-		.def(py::init<ImportFeatures>(), py::arg("import_features") = ImportFeatures::AllCurrent)
+		.def(py::init<Features, const std::string&>(), py::arg("import_features") = Features::AllCurrent, py::arg("modules_directory") = "")
 		.def("parse_file", &Yaramod::parseFile, py::arg("file_path"), py::arg("parser_mode") = ParserMode::Regular)
 		.def("parse_string", [](Yaramod& self, const std::string& str, ParserMode parserMode) {
 				std::istringstream stream(str);
 				return self.parseStream(stream, parserMode);
 			}, py::arg("str"), py::arg("parser_mode") = ParserMode::Regular)
-		.def_property_readonly("yara_file", &Yaramod::getParsedFile);
+		.def_property_readonly("yara_file", &Yaramod::getParsedFile)
+		.def_property_readonly("modules", &Yaramod::getModules);
 }
 
 PYBIND11_MODULE(yaramod, module)
